package edu.travel.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import edu.travel.entity.BtAccountInfo;
import edu.travel.mapper.BtAccountInfoMapper;
import javax.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * AccountInfoServiceImpl 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/3/13
 */

@Service
public class AccountInfoServiceImpl implements UserDetailsService {

    @Autowired
    private BtAccountInfoMapper btAccountInfoMapper;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
//    @Autowired
//    private SysRoleService roleService;
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {

            // 1. 基本校验
            if (StringUtils.isBlank(username)) {
                throw new UsernameNotFoundException("手机号不能为空");
            }

            // 2. 获取当前请求
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder
                    .currentRequestAttributes()).getRequest();

            // 3. 查询用户
            BtAccountInfo tenant = btAccountInfoMapper.selectOne(new LambdaQueryWrapper<BtAccountInfo>()
                    .eq(BtAccountInfo::getPhone, username));
            if (tenant == null) {
                throw new UsernameNotFoundException("用户不存在");
            }

            // 4. 处理不同登录方式
            String loginType = request.getParameter("loginFrom");
            if ("admin".equals(loginType)) {
                // 管理员密码登录
                String password = request.getParameter("password");
                if (!tenant.getPassword().equals(password)) {
                    throw new UsernameNotFoundException("密码错误");
                }
            } else {
                // 用户短信登录
                String code = request.getParameter("code");
                String redisKey = username + "_sms_code";
                String storedCode = redisTemplate.opsForValue().get(redisKey);

                if (!StringUtils.equals(code, storedCode)) {
                    throw new UsernameNotFoundException("验证码错误");
                }
                // 清除已用验证码
                redisTemplate.delete(redisKey);
            }

            // 5. 获取角色信息
//            List<SysRole> roles = roleService.getRoleListByUserId(tenant.getId());
//            List<String> roleNames = roles.stream()
//                    .map(SysRole::getName)
//                    .collect(Collectors.toList());

            // 6. 构建UserDetails对象
//        String[] array = roleNames.toArray(new String[0]);
        return User.builder()
                    .username(username)
                    .password(tenant.getPassword()) // 实际密码应从数据库获取
                    .roles(new String[0])
                    .build();
        }
}
