package edu.travel.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.dto.LoginDto;
import edu.travel.dto.SendSmsDto;
import edu.travel.entity.AccountInfo;
import edu.travel.entity.BtAccountInfo;
import edu.travel.mapper.BtAccountInfoMapper;
import edu.travel.service.BtAccountInfoService;
import edu.travel.util.SMSUtils;
import javax.servlet.http.HttpServletRequest;

import edu.travel.vo.BtAccountInfoVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Random;
import java.util.concurrent.TimeUnit;



@Service
public class BtAccountInfoServiceImpl extends ServiceImpl<BtAccountInfoMapper, BtAccountInfo> implements BtAccountInfoService {

//    private static final BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
    @Autowired
    private SMSUtils smsUtils;
    @Autowired
    private RedisTemplate redisTemplate;
    @Override
    public BtAccountInfoVo doLogin(LoginDto loginDto) {
//        BtAccountInfo accountInfoByPhone = getAccountInfoByPhone(accountInfo.getPhone());
//        if (accountInfoByPhone != null){
//            boolean passwordMatches = encoder.matches(accountInfo.getPassword(), accountInfoByPhone.getPassword());
//            if (passwordMatches) {
//                String token = TokenUtil.generateToken(accountInfo);
//                AccountInfo accountInfo1 = BeanUtil.copyProperties(accountInfoByPhone, AccountInfo.class);
//                accountInfo1.setToken(token);
//                return accountInfo1;
//            } else {
//                throw new RuntimeException("密码错误");
//            }
//        } else {
//            accountInfo.setPassword(encoder.encode(accountInfo.getPassword()));
//            BtAccountInfo entity = BeanUtil.copyProperties(accountInfo, BtAccountInfo.class);
//            save(entity);
//
//            AccountInfo accountInfo1 = BeanUtil.copyProperties(entity, AccountInfo.class);
//            String token = TokenUtil.generateToken(accountInfo);
//            accountInfo1.setToken(token);
//            return accountInfo1;
//        }

        String username = loginDto.getPhoneNumber();
        // 1. 基本校验
        if (ObjectUtil.isEmpty(loginDto) || StringUtils.isBlank(username) ) {
            throw new UsernameNotFoundException("手机号不能为空");
        }
        if (StringUtils.isBlank(loginDto.getCode())){
            throw new UsernameNotFoundException("验证码不能为空");
        }

        // 2. 获取当前请求
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder
                .currentRequestAttributes()).getRequest();

        // 3. 查询用户
        BtAccountInfo tenant = this.getOne(new LambdaQueryWrapper<BtAccountInfo>()
                .eq(BtAccountInfo::getPhone, username));
        if (tenant == null) {
            throw new UsernameNotFoundException("用户不存在");
        }

        // 4. 处理不同登录方式
        String loginType = request.getParameter("loginFrom");
        if ("admin".equals(loginType)) {
            // 管理员密码登录
            String password = request.getParameter("password");
            if (!tenant.getPassword().equals(password)) {
                throw new UsernameNotFoundException("密码错误");
            }
        } else {
            // 用户短信登录
            String code = loginDto.getCode();
            String redisKey = username + "_sms_code";
            String storedCode = redisTemplate.opsForValue().get(redisKey).toString();

            if (!StringUtils.equals(code, storedCode)) {
                throw new UsernameNotFoundException("验证码错误");
            }
            // 清除已用验证码
            redisTemplate.delete(redisKey);
        }
        return null;
    }

    @Override
    public void sendSms(SendSmsDto sendSmsDto) {
        if (ObjectUtil.isEmpty(sendSmsDto) || StringUtils.isBlank(sendSmsDto.getPhoneNumber())){
           throw new RuntimeException("手机号不能为空");
        }
        // 校验图形验证码
        String imageCode = (String) redisTemplate.opsForValue().get(sendSmsDto.getPhoneNumber() + "_image_code");
        //发送短信
        Object o = redisTemplate.opsForValue().get(sendSmsDto.getPhoneNumber() + "_user_sms");
        if (o!=null) throw new RuntimeException("验证码已发送,请一分钟后重试");
        //短信码
        String smsCode = String.valueOf(new Random().nextInt(9000) + 1000);
        redisTemplate.opsForValue().set(sendSmsDto.getPhoneNumber() + "_user_sms", smsCode, 1, TimeUnit.MINUTES);
        smsUtils.sendMsg(sendSmsDto.getCountryCode(), sendSmsDto.getPhoneNumber(), smsCode);
    }


    /**
     * 根据手机号查询用户
     */
    @Override
    public AccountInfo findByPhone(String phone) {
        BtAccountInfo one = this.getOne(new LambdaQueryWrapper<BtAccountInfo>().eq(BtAccountInfo::getPhone, phone).eq(BtAccountInfo::getUserStatus, 1));
        if (one != null) {
            AccountInfo accountInfo = BeanUtil.copyProperties(one, AccountInfo.class);
            return accountInfo;
        }
        return null;
    }
}
